from .base import Template
from ..utils.field import Field
from ..version import IsoVersion
from ..utils.header import NoHeaderRule
from ..utils.length import FixedLengthRule, VariableLengthRule
from ..utils.parser import ParseBin, ParseDe048, ParseDe112, ParseString, ParseBitmap, ParseBitString


mastercard = {
    IsoVersion._1987: Template(headerRule=NoHeaderRule(), fields=[
        Field(
            name="MTI",
            lengthRule=FixedLengthRule(limit=4),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE000",
            lengthRule=FixedLengthRule(limit=8),
            parsingRule=ParseBitmap(offset=0),
        ),
        Field(
            name="DE001",
            lengthRule=FixedLengthRule(limit=8),
            parsingRule=ParseBitmap(offset=64),
        ),
        Field(
            name="DE002",
            lengthRule=VariableLengthRule(limit=19, byteLength=2),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE003",
            lengthRule=FixedLengthRule(limit=6),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE004",
            lengthRule=FixedLengthRule(limit=12),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE005",
            lengthRule=FixedLengthRule(limit=12),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE006",
            lengthRule=FixedLengthRule(limit=12),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE007",
            lengthRule=FixedLengthRule(limit=10),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE008",
            lengthRule=FixedLengthRule(limit=8),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE009",
            lengthRule=FixedLengthRule(limit=8),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE010",
            lengthRule=FixedLengthRule(limit=8),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE011",
            lengthRule=FixedLengthRule(limit=6),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE012",
            lengthRule=FixedLengthRule(limit=6),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE013",
            lengthRule=FixedLengthRule(limit=4),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE014",
            lengthRule=FixedLengthRule(limit=4),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE015",
            lengthRule=FixedLengthRule(limit=4),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE016",
            lengthRule=FixedLengthRule(limit=4),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE017",
            lengthRule=FixedLengthRule(limit=4),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE018",
            lengthRule=FixedLengthRule(limit=4),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE019",
            lengthRule=FixedLengthRule(limit=3),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE020",
            lengthRule=FixedLengthRule(limit=3),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE021",
            lengthRule=FixedLengthRule(limit=3),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE022",
            lengthRule=FixedLengthRule(limit=3),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE023",
            lengthRule=FixedLengthRule(limit=3),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE024",
            lengthRule=FixedLengthRule(limit=3),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE025",
            lengthRule=FixedLengthRule(limit=2),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE026",
            lengthRule=FixedLengthRule(limit=2),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE027",
            lengthRule=FixedLengthRule(limit=1),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE028",
            lengthRule=FixedLengthRule(limit=9),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE029",
            lengthRule=FixedLengthRule(limit=9),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE030",
            lengthRule=FixedLengthRule(limit=9),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE031",
            lengthRule=FixedLengthRule(limit=9),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE032",
            lengthRule=VariableLengthRule(limit=11, byteLength=2),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE033",
            lengthRule=VariableLengthRule(limit=11, byteLength=2),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE034",
            lengthRule=VariableLengthRule(limit=28, byteLength=2),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE035",
            lengthRule=VariableLengthRule(limit=37, byteLength=2),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE036",
            lengthRule=VariableLengthRule(limit=104, byteLength=3),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE037",
            lengthRule=FixedLengthRule(limit=12),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE038",
            lengthRule=FixedLengthRule(limit=6),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE039",
            lengthRule=FixedLengthRule(limit=2),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE040",
            lengthRule=FixedLengthRule(limit=3),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE041",
            lengthRule=FixedLengthRule(limit=8),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE042",
            lengthRule=FixedLengthRule(limit=15),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE043",
            lengthRule=FixedLengthRule(limit=40),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE044",
            lengthRule=VariableLengthRule(limit=25, byteLength=2),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE045",
            lengthRule=VariableLengthRule(limit=76, byteLength=2),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE046",
            lengthRule=VariableLengthRule(limit=999, byteLength=3),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE047",
            lengthRule=VariableLengthRule(limit=999, byteLength=3),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE048",
            lengthRule=VariableLengthRule(limit=999, byteLength=3),
            parsingRule=ParseDe048(),
        ),
        Field(
            name="DE049",
            lengthRule=FixedLengthRule(limit=3),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE050",
            lengthRule=FixedLengthRule(limit=3),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE051",
            lengthRule=FixedLengthRule(limit=3),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE052",
            lengthRule=FixedLengthRule(limit=8),
            parsingRule=ParseBitString(),
        ),
        Field(
            name="DE053",
            lengthRule=FixedLengthRule(limit=16),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE054",
            lengthRule=VariableLengthRule(limit=120, byteLength=3),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE055",
            lengthRule=VariableLengthRule(limit=999, byteLength=3),
            parsingRule=ParseBin(),
        ),
        Field(
            name="DE056",
            lengthRule=VariableLengthRule(limit=999, byteLength=3),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE057",
            lengthRule=VariableLengthRule(limit=999, byteLength=3),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE058",
            lengthRule=VariableLengthRule(limit=999, byteLength=3),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE059",
            lengthRule=VariableLengthRule(limit=999, byteLength=3),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE060",
            lengthRule=VariableLengthRule(limit=999, byteLength=3),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE061",
            lengthRule=VariableLengthRule(limit=999, byteLength=3),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE062",
            lengthRule=VariableLengthRule(limit=999, byteLength=3),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE063",
            lengthRule=VariableLengthRule(limit=999, byteLength=3),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE064",
            lengthRule=FixedLengthRule(limit=8),
            parsingRule=ParseBitString(),
        ),
        Field(
            name="DE065",
            lengthRule=FixedLengthRule(limit=8),
            parsingRule=ParseBitmap(offset=128),
        ),
        Field(
            name="DE066",
            lengthRule=FixedLengthRule(limit=1),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE067",
            lengthRule=FixedLengthRule(limit=2),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE068",
            lengthRule=FixedLengthRule(limit=3),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE069",
            lengthRule=FixedLengthRule(limit=3),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE070",
            lengthRule=FixedLengthRule(limit=3),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE071",
            lengthRule=FixedLengthRule(limit=4),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE072",
            lengthRule=FixedLengthRule(limit=4),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE073",
            lengthRule=FixedLengthRule(limit=6),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE074",
            lengthRule=FixedLengthRule(limit=10),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE075",
            lengthRule=FixedLengthRule(limit=10),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE076",
            lengthRule=FixedLengthRule(limit=10),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE077",
            lengthRule=FixedLengthRule(limit=10),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE078",
            lengthRule=FixedLengthRule(limit=10),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE079",
            lengthRule=FixedLengthRule(limit=10),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE080",
            lengthRule=FixedLengthRule(limit=10),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE081",
            lengthRule=FixedLengthRule(limit=10),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE082",
            lengthRule=FixedLengthRule(limit=12),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE083",
            lengthRule=FixedLengthRule(limit=12),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE084",
            lengthRule=FixedLengthRule(limit=12),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE085",
            lengthRule=FixedLengthRule(limit=12),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE086",
            lengthRule=FixedLengthRule(limit=16),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE087",
            lengthRule=FixedLengthRule(limit=16),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE088",
            lengthRule=FixedLengthRule(limit=16),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE089",
            lengthRule=FixedLengthRule(limit=16),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE090",
            lengthRule=FixedLengthRule(limit=42),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE091",
            lengthRule=FixedLengthRule(limit=1),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE092",
            lengthRule=FixedLengthRule(limit=2),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE093",
            lengthRule=FixedLengthRule(limit=5),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE094",
            lengthRule=FixedLengthRule(limit=7),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE095",
            lengthRule=FixedLengthRule(limit=42),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE096",
            lengthRule=FixedLengthRule(limit=8),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE097",
            lengthRule=FixedLengthRule(limit=17),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE098",
            lengthRule=FixedLengthRule(limit=25),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE099",
            lengthRule=VariableLengthRule(limit=11, byteLength=2),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE100",
            lengthRule=VariableLengthRule(limit=11, byteLength=2),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE101",
            lengthRule=VariableLengthRule(limit=17, byteLength=2),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE102",
            lengthRule=VariableLengthRule(limit=28, byteLength=2),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE103",
            lengthRule=VariableLengthRule(limit=28, byteLength=2),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE104",
            lengthRule=VariableLengthRule(limit=100, byteLength=3),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE105",
            lengthRule=VariableLengthRule(limit=999, byteLength=3),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE106",
            lengthRule=VariableLengthRule(limit=999, byteLength=3),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE107",
            lengthRule=VariableLengthRule(limit=999, byteLength=3),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE108",
            lengthRule=VariableLengthRule(limit=999, byteLength=3),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE109",
            lengthRule=VariableLengthRule(limit=999, byteLength=3),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE110",
            lengthRule=VariableLengthRule(limit=999, byteLength=3),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE111",
            lengthRule=VariableLengthRule(limit=999, byteLength=3),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE112",
            lengthRule=VariableLengthRule(limit=999, byteLength=3),
            parsingRule=ParseDe112(),
        ),
        Field(
            name="DE113",
            lengthRule=VariableLengthRule(limit=999, byteLength=3),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE114",
            lengthRule=VariableLengthRule(limit=999, byteLength=3),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE115",
            lengthRule=VariableLengthRule(limit=999, byteLength=3),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE116",
            lengthRule=VariableLengthRule(limit=999, byteLength=3),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE117",
            lengthRule=VariableLengthRule(limit=999, byteLength=3),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE118",
            lengthRule=VariableLengthRule(limit=999, byteLength=3),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE119",
            lengthRule=VariableLengthRule(limit=999, byteLength=3),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE120",
            lengthRule=VariableLengthRule(limit=999, byteLength=3),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE121",
            lengthRule=VariableLengthRule(limit=999, byteLength=3),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE122",
            lengthRule=VariableLengthRule(limit=999, byteLength=3),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE123",
            lengthRule=VariableLengthRule(limit=999, byteLength=3),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE124",
            lengthRule=VariableLengthRule(limit=999, byteLength=3),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE125",
            lengthRule=VariableLengthRule(limit=999, byteLength=3),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE126",
            lengthRule=VariableLengthRule(limit=999, byteLength=3),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE127",
            lengthRule=VariableLengthRule(limit=999, byteLength=3),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE128",
            lengthRule=FixedLengthRule(limit=8),
            parsingRule=ParseBitString(),
        ),
    ]),
    IsoVersion._1993: Template(headerRule=NoHeaderRule(), fields=[
        Field(
            name="MTI",
            lengthRule=FixedLengthRule(limit=8),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE000",
            lengthRule=FixedLengthRule(limit=8),
            parsingRule=ParseBitmap(offset=0),
        ),
        Field(
            name="DE001",
            lengthRule=FixedLengthRule(limit=8),
            parsingRule=ParseBitmap(offset=64),
        ),
        Field(
            name="DE002",
            lengthRule=VariableLengthRule(limit=19, byteLength=2),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE003",
            lengthRule=FixedLengthRule(limit=6),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE004",
            lengthRule=FixedLengthRule(limit=12),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE005",
            lengthRule=FixedLengthRule(limit=12),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE006",
            lengthRule=FixedLengthRule(limit=12),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE007",
            lengthRule=FixedLengthRule(limit=10),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE008",
            lengthRule=FixedLengthRule(limit=8),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE009",
            lengthRule=FixedLengthRule(limit=8),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE010",
            lengthRule=FixedLengthRule(limit=8),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE011",
            lengthRule=FixedLengthRule(limit=6),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE012",
            lengthRule=FixedLengthRule(limit=12),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE013",
            lengthRule=FixedLengthRule(limit=4),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE014",
            lengthRule=FixedLengthRule(limit=4),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE015",
            lengthRule=FixedLengthRule(limit=4),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE016",
            lengthRule=FixedLengthRule(limit=4),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE017",
            lengthRule=FixedLengthRule(limit=4),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE018",
            lengthRule=FixedLengthRule(limit=4),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE019",
            lengthRule=FixedLengthRule(limit=3),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE020",
            lengthRule=FixedLengthRule(limit=3),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE021",
            lengthRule=FixedLengthRule(limit=3),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE022",
            lengthRule=FixedLengthRule(limit=12),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE023",
            lengthRule=FixedLengthRule(limit=3),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE024",
            lengthRule=FixedLengthRule(limit=3),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE025",
            lengthRule=FixedLengthRule(limit=4),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE026",
            lengthRule=FixedLengthRule(limit=4),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE027",
            lengthRule=FixedLengthRule(limit=1),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE028",
            lengthRule=FixedLengthRule(limit=9),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE029",
            lengthRule=FixedLengthRule(limit=9),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE030",
            lengthRule=FixedLengthRule(limit=24),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE031",
            lengthRule=VariableLengthRule(limit=23, byteLength=2),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE032",
            lengthRule=VariableLengthRule(limit=11, byteLength=2),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE033",
            lengthRule=VariableLengthRule(limit=11, byteLength=2),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE034",
            lengthRule=VariableLengthRule(limit=28, byteLength=2),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE035",
            lengthRule=VariableLengthRule(limit=37, byteLength=2),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE036",
            lengthRule=VariableLengthRule(limit=104, byteLength=3),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE037",
            lengthRule=FixedLengthRule(limit=12),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE038",
            lengthRule=FixedLengthRule(limit=6),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE039",
            lengthRule=FixedLengthRule(limit=2),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE040",
            lengthRule=FixedLengthRule(limit=3),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE041",
            lengthRule=FixedLengthRule(limit=8),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE042",
            lengthRule=FixedLengthRule(limit=15),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE043",
            lengthRule=VariableLengthRule(limit=99, byteLength=2),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE044",
            lengthRule=VariableLengthRule(limit=25, byteLength=2),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE045",
            lengthRule=VariableLengthRule(limit=76, byteLength=2),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE046",
            lengthRule=VariableLengthRule(limit=999, byteLength=3),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE047",
            lengthRule=VariableLengthRule(limit=999, byteLength=3),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE048",
            lengthRule=VariableLengthRule(limit=999, byteLength=3),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE049",
            lengthRule=FixedLengthRule(limit=3),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE050",
            lengthRule=FixedLengthRule(limit=3),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE051",
            lengthRule=FixedLengthRule(limit=3),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE052",
            lengthRule=FixedLengthRule(limit=8),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE053",
            lengthRule=FixedLengthRule(limit=16),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE054",
            lengthRule=VariableLengthRule(limit=240, byteLength=3),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE055",
            lengthRule=VariableLengthRule(limit=999, byteLength=3),
            parsingRule=ParseBin(),
        ),
        Field(
            name="DE056",
            lengthRule=VariableLengthRule(limit=999, byteLength=3),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE057",
            lengthRule=VariableLengthRule(limit=999, byteLength=3),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE058",
            lengthRule=VariableLengthRule(limit=999, byteLength=3),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE059",
            lengthRule=VariableLengthRule(limit=999, byteLength=3),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE060",
            lengthRule=VariableLengthRule(limit=999, byteLength=3),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE061",
            lengthRule=VariableLengthRule(limit=999, byteLength=3),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE062",
            lengthRule=VariableLengthRule(limit=999, byteLength=3),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE063",
            lengthRule=VariableLengthRule(limit=16, byteLength=3),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE064",
            lengthRule=FixedLengthRule(limit=8),
            parsingRule=ParseBitString(),
        ),
        Field(
            name="DE065",
            lengthRule=FixedLengthRule(limit=8),
            parsingRule=ParseBitmap(offset=128),
        ),
        Field(
            name="DE066",
            lengthRule=FixedLengthRule(limit=1),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE067",
            lengthRule=FixedLengthRule(limit=2),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE068",
            lengthRule=FixedLengthRule(limit=3),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE069",
            lengthRule=FixedLengthRule(limit=3),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE070",
            lengthRule=FixedLengthRule(limit=3),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE071",
            lengthRule=FixedLengthRule(limit=8),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE072",
            lengthRule=VariableLengthRule(limit=999, byteLength=3),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE073",
            lengthRule=FixedLengthRule(limit=6),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE074",
            lengthRule=FixedLengthRule(limit=10),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE075",
            lengthRule=FixedLengthRule(limit=10),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE076",
            lengthRule=FixedLengthRule(limit=10),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE077",
            lengthRule=FixedLengthRule(limit=10),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE078",
            lengthRule=FixedLengthRule(limit=10),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE079",
            lengthRule=FixedLengthRule(limit=10),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE080",
            lengthRule=FixedLengthRule(limit=10),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE081",
            lengthRule=FixedLengthRule(limit=10),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE082",
            lengthRule=FixedLengthRule(limit=12),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE083",
            lengthRule=FixedLengthRule(limit=12),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE084",
            lengthRule=FixedLengthRule(limit=12),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE085",
            lengthRule=FixedLengthRule(limit=12),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE086",
            lengthRule=FixedLengthRule(limit=16),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE087",
            lengthRule=FixedLengthRule(limit=16),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE088",
            lengthRule=FixedLengthRule(limit=16),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE089",
            lengthRule=FixedLengthRule(limit=16),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE090",
            lengthRule=FixedLengthRule(limit=42),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE091",
            lengthRule=FixedLengthRule(limit=1),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE092",
            lengthRule=FixedLengthRule(limit=2),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE093",
            lengthRule=VariableLengthRule(limit=11, byteLength=2),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE094",
            lengthRule=VariableLengthRule(limit=11, byteLength=2),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE095",
            lengthRule=VariableLengthRule(limit=10, byteLength=2),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE096",
            lengthRule=FixedLengthRule(limit=8),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE097",
            lengthRule=FixedLengthRule(limit=17),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE098",
            lengthRule=FixedLengthRule(limit=25),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE099",
            lengthRule=VariableLengthRule(limit=11, byteLength=2),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE100",
            lengthRule=VariableLengthRule(limit=11, byteLength=2),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE101",
            lengthRule=VariableLengthRule(limit=17, byteLength=2),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE102",
            lengthRule=VariableLengthRule(limit=28, byteLength=2),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE103",
            lengthRule=VariableLengthRule(limit=28, byteLength=2),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE104",
            lengthRule=VariableLengthRule(limit=100, byteLength=3),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE105",
            lengthRule=VariableLengthRule(limit=999, byteLength=3),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE106",
            lengthRule=VariableLengthRule(limit=999, byteLength=3),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE107",
            lengthRule=VariableLengthRule(limit=999, byteLength=3),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE108",
            lengthRule=VariableLengthRule(limit=999, byteLength=3),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE109",
            lengthRule=VariableLengthRule(limit=999, byteLength=3),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE110",
            lengthRule=VariableLengthRule(limit=999, byteLength=3),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE111",
            lengthRule=VariableLengthRule(limit=12, byteLength=3),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE112",
            lengthRule=VariableLengthRule(limit=999, byteLength=3),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE113",
            lengthRule=VariableLengthRule(limit=999, byteLength=3),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE114",
            lengthRule=VariableLengthRule(limit=999, byteLength=3),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE115",
            lengthRule=VariableLengthRule(limit=999, byteLength=3),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE116",
            lengthRule=VariableLengthRule(limit=999, byteLength=3),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE117",
            lengthRule=VariableLengthRule(limit=999, byteLength=3),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE118",
            lengthRule=VariableLengthRule(limit=999, byteLength=3),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE119",
            lengthRule=VariableLengthRule(limit=999, byteLength=3),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE120",
            lengthRule=VariableLengthRule(limit=999, byteLength=3),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE121",
            lengthRule=VariableLengthRule(limit=999, byteLength=3),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE122",
            lengthRule=VariableLengthRule(limit=999, byteLength=3),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE123",
            lengthRule=VariableLengthRule(limit=999, byteLength=3),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE124",
            lengthRule=VariableLengthRule(limit=999, byteLength=3),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE125",
            lengthRule=VariableLengthRule(limit=999, byteLength=3),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE126",
            lengthRule=VariableLengthRule(limit=999, byteLength=3),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE127",
            lengthRule=VariableLengthRule(limit=999, byteLength=3),
            parsingRule=ParseString(),
        ),
        Field(
            name="DE128",
            lengthRule=FixedLengthRule(limit=8),
            parsingRule=ParseBitString(),
        ),
    ]),
}
